package SPVM::Double;

1;

=head1 NAME

SPVM::Double - Double object

=head1 CLASS METHODS

=head2 new

  sub new : Double ($value : double)

Construct Double object with specified value.

=head1 METHODS

=head2 get_value

  sub get_value : double ($self : self)

Get value.

=head2 set_value

  sub set_value : void ($self : self, $value : double)

Set value.
