package SPVM::Int;

1;

=head1 NAME

SPVM::Int - Int object

=head1 CLASS METHODS

=head2 new

  sub new : Int ($value : int)

Construct Int object with specified value.

=head1 METHODS

=head2 get_value

  sub get_value : int ($self : self)

Get value.

=head2 set_value

  sub set_value : void ($self : self, $value : int)

Set value.
