package SPVM::Short;

1;

=head1 NAME

SPVM::Short - Short object

=head1 CLASS METHODS

=head2 new

  sub new : Short ($value : short)

Construct Short object with specified value.

=head1 METHODS

=head2 get_value

  sub get_value : short ($self : self)

Get value.

=head2 set_value

  sub set_value : void ($self : self, $value : short)

Set value.
